@@ -180,7 +180,7 @@ def _get_namespace(tag):
     return namespace
 
 
-def _extract_pages(f):
+def _extract_pages(f, filter_namespaces=False):
     """
     Extract pages from MediaWiki database dump.
 
@@ -201,12 +201,20 @@ def _extract_pages(f):
     page_tag = "{%(ns)s}page" % ns_mapping
     text_path = "./{%(ns)s}revision/{%(ns)s}text" % ns_mapping
     title_path = "./{%(ns)s}title" % ns_mapping
+    ns_path = "./{%(ns)s}ns" % ns_mapping
+    pageid_path = "./{%(ns)s}id" % ns_mapping
 
     for elem in elems:
         if elem.tag == page_tag:
             title = elem.find(title_path).text
             text = elem.find(text_path).text
-            yield title, text or ""     # empty page will yield None
+
+            ns = elem.find(ns_path).text
+            if filter_namespaces and ns not in filter_namespaces:
+                text = None
+
+            pageid = elem.find(pageid_path).text
+            yield title, text or "", pageid     # empty page will yield None
 
             # Prune the element tree, as per
             # http://www.ibm.com/developerworks/xml/library/x-hiperfparse/
@@ -223,19 +231,19 @@ def process_article(args):
     Parse a wikipedia article, returning its content as a list of tokens
     (utf8-encoded strings).
     """
-    text, lemmatize = args
+    text, lemmatize, title, pageid = args
     text = filter_wiki(text)
     if lemmatize:
         result = utils.lemmatize(text)
     else:
         result = tokenize(text)
-    return result
+    return result, title, pageid
 
 
 
 class WikiCorpus(TextCorpus):
     """
-    Treat a wikipedia articles dump (*articles.xml.bz2) as a (read-only) corpus.
+    Treat a wikipedia articles dump (\*articles.xml.bz2) as a (read-only) corpus.
 
     The documents are extracted on-the-fly, so that the whole (massive) dump
     can stay compressed on disk.
@@ -244,7 +252,7 @@ class WikiCorpus(TextCorpus):
     >>> wiki.saveAsText('wiki_en_vocab200k') # another 8h, creates a file in MatrixMarket format plus file with id->word
 
     """
-    def __init__(self, fname, processes=None, lemmatize=utils.HAS_PATTERN, dictionary=None):
+    def __init__(self, fname, processes=None, lemmatize=utils.HAS_PATTERN, dictionary=None, filter_namespaces=('0',)):
         """
         Initialize the corpus. Unless a dictionary is provided, this scans the
         corpus once, to determine its vocabulary.
@@ -255,6 +263,8 @@ class WikiCorpus(TextCorpus):
 
         """
         self.fname = fname
+        self.filter_namespaces = filter_namespaces
+        self.metadata = False
         if processes is None:
             processes = max(1, multiprocessing.cpu_count() - 1)
         self.processes = processes
@@ -277,22 +287,25 @@ class WikiCorpus(TextCorpus):
         the standard corpus interface instead of this function::
 
         >>> for vec in wiki_corpus:
-        >>>     print vec
+        >>>     print(vec)
         """
         articles, articles_all = 0, 0
         positions, positions_all = 0, 0
-        texts = ((text, self.lemmatize) for _, text in _extract_pages(bz2.BZ2File(self.fname)))
+        texts = ((text, self.lemmatize, title, pageid) for title, text, pageid in _extract_pages(bz2.BZ2File(self.fname), self.filter_namespaces))
         pool = multiprocessing.Pool(self.processes)
         # process the corpus in smaller chunks of docs, because multiprocessing.Pool
         # is dumb and would load the entire input into RAM at once...
         for group in utils.chunkize(texts, chunksize=10 * self.processes, maxsize=1):
-            for tokens in pool.imap(process_article, group): # chunksize=10):
+            for tokens, title, pageid in pool.imap(process_article, group): # chunksize=10):
                 articles_all += 1
                 positions_all += len(tokens)
                 if len(tokens) > ARTICLE_MIN_WORDS: # article redirects and short stubs are pruned here
                     articles += 1
                     positions += len(tokens)
-                    yield tokens
+                    if self.metadata:
+                        yield (tokens, (pageid, title))
+                    else:
+                        yield tokens
         pool.terminate()
 
         logger.info("finished iterating over Wikipedia corpus of %i documents with %i positions"
