@@ -50,7 +50,7 @@ except ImportError: # maxentropy has been removed for next release
 
 
 from gensim import interfaces, utils
-from gensim._six.moves import xrange
+from six.moves import xrange
 
 
 def dirichlet_expectation(alpha):
@@ -391,8 +391,8 @@ class LdaModel(interfaces.TransformationABC):
                 sstats[:, ids] += numpy.outer(expElogthetad.T, cts / phinorm)
 
         if len(chunk) > 1:
-            logger.info("%i/%i documents converged within %i iterations" %
-                         (converged, len(chunk), self.iterations))
+            logger.debug("%i/%i documents converged within %i iterations" %
+                (converged, len(chunk), self.iterations))
 
         if collect_sstats:
             # This step finishes computing the sufficient statistics for the
@@ -518,7 +518,7 @@ class LdaModel(interfaces.TransformationABC):
         logger.info("running %s LDA training, %s topics, %i passes over "
                     "the supplied corpus of %i documents, updating model once "
                     "every %i documents, evaluating perplexity every %i documents, "
-                    "iterating %i with a convergence threshold of %i" %
+                    "iterating %ix with a convergence threshold of %f" %
                     (updatetype, self.num_topics, passes, lencorpus,
                         updateafter, evalafter, iterations,
                         gamma_threshold))
@@ -655,14 +655,17 @@ class LdaModel(interfaces.TransformationABC):
 
     def show_topics(self, topics=10, topn=10, log=False, formatted=True):
         """
-        Print the `topN` most probable words for `topics` number of topics.
-        Set `topics=-1` to print all topics.
+        For `topics` number of topics, return `topn` most significant words
+        (10 words per topic, by default).
 
-        Unlike LSA, there is no ordering between the topics in LDA.
-        The printed `topics <= self.num_topics` subset of all topics is therefore
-        arbitrary and may change between two runs.
+        The topics are returned as a list -- a list of strings if `formatted` is
+        True, or a list of (probability, word) 2-tuples if False.
 
-        Set `formatted=True` to return the topics as a list of strings, or `False` as lists of (weight, word) pairs.
+        If `log` is True, also output this result to log.
+
+        Unlike LSA, there is no natural ordering between the topics in LDA.
+        The returned `topics <= self.num_topics` subset of all topics is therefore
+        arbitrary and may change between two LDA training runs.
 
         """
         if topics < 0 or topics >= self.num_topics:
