@@ -6,14 +6,15 @@
 
 
 """
-Deep learning via word2vec's "hierarchical softmax skip-gram and CBOW models" [1]_.
+Deep learning via word2vec's "skip-gram and CBOW models", using either
+hierarchical softmax or negative sampling [1]_ [2]_.
 
-The training algorithm was originally ported from the C package https://code.google.com/p/word2vec/
+The training algorithms were originally ported from the C package https://code.google.com/p/word2vec/
 and extended with additional functionality.
 
 For a blog tutorial on gensim word2vec, with an interactive web app trained on GoogleNews, visit http://radimrehurek.com/2014/02/word2vec-tutorial/
 
-**Install Cython with `pip install cython` to use optimized word2vec training** (70x speedup [2]_).
+**Install Cython with `pip install cython` to use optimized word2vec training** (70x speedup [3]_).
 
 Initialize a model with e.g.::
 
@@ -53,7 +54,9 @@ If you're finished training a model (=no more updates, only querying), you can d
 to trim unneeded model memory = use (much) less RAM.
 
 .. [1] Tomas Mikolov, Kai Chen, Greg Corrado, and Jeffrey Dean. Efficient Estimation of Word Representations in Vector Space. In Proceedings of Workshop at ICLR, 2013.
-.. [2] Optimizing word2vec in gensim, http://radimrehurek.com/2013/09/word2vec-in-python-part-two-optimizing/
+.. [2] Tomas Mikolov, Ilya Sutskever, Kai Chen, Greg Corrado, and Jeffrey Dean. Distributed Representations of Words and Phrases and their Compositionality.
+       In Proceedings of NIPS, 2013.
+.. [3] Optimizing word2vec in gensim, http://radimrehurek.com/2013/09/word2vec-in-python-part-two-optimizing/
 """
 
 import logging
@@ -61,6 +64,7 @@ import sys
 import os
 import heapq
 import time
+from copy import deepcopy
 import threading
 try:
     from queue import Queue
@@ -68,98 +72,134 @@ except ImportError:
     from Queue import Queue
 
 from numpy import exp, dot, zeros, outer, random, dtype, get_include, float32 as REAL,\
-    uint32, seterr, array, uint8, vstack, argsort, fromstring, sqrt, newaxis, ndarray, empty
+    uint32, seterr, array, uint8, vstack, argsort, fromstring, sqrt, newaxis, ndarray, empty, sum as np_sum
 
 logger = logging.getLogger("gensim.models.word2vec")
 
 
 from gensim import utils, matutils  # utility fnc for pickling, common scipy operations etc
-from gensim._six import iteritems, itervalues, string_types
-from gensim._six.moves import xrange
+from six import iteritems, itervalues, string_types
+from six.moves import xrange
 
 
 try:
-    # try to compile and use the faster cython version
-    import pyximport
-    pyximport.install(setup_args={"include_dirs": get_include()})
-    from word2vec_inner import train_sentence_sg, train_sentence_cbow, FAST_VERSION
-except:
-    # failed... fall back to plain numpy (20-80x slower training than the above)
-    FAST_VERSION = -1
+    from gensim_addons.models.word2vec_inner import train_sentence_sg, train_sentence_cbow, FAST_VERSION
+except ImportError:
+    try:
+        # try to compile and use the faster cython version
+        import pyximport
+        models_dir = os.path.dirname(__file__) or os.getcwd()
+        pyximport.install(setup_args={"include_dirs": [models_dir, get_include()]})
+        from word2vec_inner import train_sentence_sg, train_sentence_cbow, FAST_VERSION
+    except:
+        # failed... fall back to plain numpy (20-80x slower training than the above)
+        FAST_VERSION = -1
 
-    def train_sentence_sg(model, sentence, alpha, work=None):
-        """
-        Update skip-gram hierarchical softmax model by training on a single sentence.
+        def train_sentence_sg(model, sentence, alpha, work=None):
+            """
+            Update skip-gram model by training on a single sentence.
 
-        The sentence is a list of Vocab objects (or None, where the corresponding
-        word is not in the vocabulary. Called internally from `Word2Vec.train()`.
+            The sentence is a list of Vocab objects (or None, where the corresponding
+            word is not in the vocabulary. Called internally from `Word2Vec.train()`.
 
-        """
-        for pos, word in enumerate(sentence):
-            if word is None:
-                continue  # OOV word in the input sentence => skip
-            reduced_window = random.randint(model.window)  # `b` in the original word2vec code
+            This is the non-optimized, Python version. If you have cython installed, gensim
+            will use the optimized version from word2vec_inner instead.
 
-            # now go over all words from the (reduced) window, predicting each one in turn
-            start = max(0, pos - model.window + reduced_window)
-            for pos2, word2 in enumerate(sentence[start : pos + model.window + 1 - reduced_window], start):
-                if pos2 == pos or word2 is None:
+            """
+            if model.negative:
+                # precompute negative labels
+                labels = zeros(model.negative + 1)
+                labels[0] = 1.0
+
+            for pos, word in enumerate(sentence):
+                if word is None:
+                    continue  # OOV word in the input sentence => skip
+                reduced_window = random.randint(model.window)  # `b` in the original word2vec code
+
+                # now go over all words from the (reduced) window, predicting each one in turn
+                start = max(0, pos - model.window + reduced_window)
+                for pos2, word2 in enumerate(sentence[start : pos + model.window + 1 - reduced_window], start):
                     # don't train on OOV words and on the `word` itself
-                    continue
+                    if word2 and not (pos2 == pos):
+                        l1 = model.syn0[word2.index]
+                        neu1e = zeros(l1.shape)
 
-                l1 = model.syn0[word2.index]
-                # work on the entire tree at once, to push as much work into numpy's C routines as possible (performance)
-                l2a = model.syn1[word.point]  # 2d matrix, codelen x layer1_size
-                fa = 1.0 / (1.0 + exp(-dot(l1, l2a.T)))  #  propagate hidden -> output
-                ga = (1 - word.code - fa) * alpha  # vector of error gradients multiplied by the learning rate
-                model.syn1[word.point] += outer(ga, l1)  # learn hidden -> output
+                        if model.hs:
+                            # work on the entire tree at once, to push as much work into numpy's C routines as possible (performance)
+                            l2a = deepcopy(model.syn1[word.point])  # 2d matrix, codelen x layer1_size
+                            fa = 1.0 / (1.0 + exp(-dot(l1, l2a.T)))  #  propagate hidden -> output
+                            ga = (1 - word.code - fa) * alpha  # vector of error gradients multiplied by the learning rate
+                            model.syn1[word.point] += outer(ga, l1)  # learn hidden -> output
+                            neu1e += dot(ga, l2a) # save error
 
-                # TODO add negative sampling?
+                        if model.negative:
+                            # use this word (label = 1) + `negative` other random words not from this sentence (label = 0)
+                            word_indices = [word.index]
+                            while len(word_indices) < model.negative + 1:
+                                w = model.table[random.randint(model.table.shape[0])]
+                                if w != word.index:
+                                    word_indices.append(w)
+                            l2b = model.syn1neg[word_indices] # 2d matrix, k+1 x layer1_size
+                            fb = 1. / (1. + exp(-dot(l1, l2b.T))) # propagate hidden -> output
+                            gb = (labels - fb) * alpha # vector of error gradients multiplied by the learning rate
+                            model.syn1neg[word_indices] += outer(gb, l1) # learn hidden -> output
+                            neu1e += dot(gb, l2b) # save error
 
-                l1 += dot(ga, l2a)  # learn input -> hidden
+                        model.syn0[word2.index] += neu1e  # learn input -> hidden
 
-        return len([word for word in sentence if word is not None])
+            return len([word for word in sentence if word is not None])
 
-    def train_sentence_cbow(model, sentence, alpha, work=None, neu1=None):
-        """
-        Update CBOW hierarchical softmax model by training on a single sentence.
+        def train_sentence_cbow(model, sentence, alpha, work=None, neu1=None):
+            """
+            Update CBOW model by training on a single sentence.
 
-        The sentence is a list of Vocab objects (or None, where the corresponding
-        word is not in the vocabulary. Called internally from `Word2Vec.train()`.
+            The sentence is a list of Vocab objects (or None, where the corresponding
+            word is not in the vocabulary. Called internally from `Word2Vec.train()`.
 
-        """
+            This is the non-optimized, Python version. If you have cython installed, gensim
+            will use the optimized version from word2vec_inner instead.
 
-        for pos, word in enumerate(sentence):
-            if word is None:
-                continue  # OOV word in the input sentence => skip
-            reduced_window = random.randint(model.window)  # `b` in the original word2vec code
+            """
+            if model.negative:
+                # precompute negative labels
+                labels = zeros(model.negative + 1)
+                labels[0] = 1.
 
-            # Combine all context words into an appropriate input
-            start = max(0, pos - model.window + reduced_window)
-            l1 = matutils.zeros_aligned((model.layer1_size), dtype=REAL)
-            count = 0
-            for pos2, word2 in enumerate(sentence[start : pos + model.window + 1 - reduced_window], start):
-                if pos2 == pos or word2 is None:
-                    pass
-                else:
-                    count += 1
-                    l1 += model.syn0[word2.index]
+            for pos, word in enumerate(sentence):
+                if word is None:
+                    continue  # OOV word in the input sentence => skip
+                reduced_window = random.randint(model.window) # `b` in the original word2vec code
+                start = max(0, pos - model.window + reduced_window)
+                window_pos = enumerate(sentence[start : pos + model.window + 1 - reduced_window], start)
+                word2_indices = [word2.index for pos2, word2 in window_pos if (word2 is not None and pos2 != pos)]
+                l1 = np_sum(model.syn0[word2_indices], axis=0) # 1 x layer1_size
+                if word2_indices and model.cbow_mean:
+                    l1 /= len(word2_indices)
+                neu1e = zeros(l1.shape)
 
-            if count > 0:
-                l1 = l1 / count
+                if model.hs:
+                    l2a = model.syn1[word.point] # 2d matrix, codelen x layer1_size
+                    fa = 1. / (1. + exp(-dot(l1, l2a.T))) # propagate hidden -> output
+                    ga = (1. - word.code - fa) * alpha # vector of error gradients multiplied by the learning rate
+                    model.syn1[word.point] += outer(ga, l1) # learn hidden -> output
+                    neu1e += dot(ga, l2a) # save error
 
-            l2a = model.syn1[word.point]  # 2d matrix, codelen x layer1_size
-            fa = 1.0 / (1.0 + exp(-dot(l1, l2a.T)))  #  propagate hidden -> output
-            ga = (1 - word.code - fa) * alpha  # vector of error gradients multiplied by the learning rate
-            model.syn1[word.point] += outer(ga, l1)  # learn hidden -> output
+                if model.negative:
+                    # use this word (label = 1) + `negative` other random words not from this sentence (label = 0)
+                    word_indices = [word.index]
+                    while len(word_indices) < model.negative + 1:
+                        w = model.table[random.randint(model.table.shape[0])]
+                        if w != word.index:
+                            word_indices.append(w)
+                    l2b = model.syn1neg[word_indices] # 2d matrix, k+1 x layer1_size
+                    fb = 1. / (1. + exp(-dot(l1, l2b.T))) # propagate hidden -> output
+                    gb = (labels - fb) * alpha # vector of error gradients multiplied by the learning rate
+                    model.syn1neg[word_indices] += outer(gb, l1) # learn hidden -> output
+                    neu1e += dot(gb, l2b) # save error
 
-            for pos2, word2 in enumerate(sentence[start : pos + model.window + 1 - reduced_window], start):
-                if pos2 == pos or word2 is None:
-                    pass
-                else:
-                    model.syn0[word2.index] += dot(ga, l2a)
+                model.syn0[word2_indices] += neu1e # learn input -> hidden, here for all words in the window separately
 
-        return len([word for word in sentence if word is not None])
+            return len([word for word in sentence if word is not None])
 
 
 class Vocab(object):
@@ -184,10 +224,11 @@ class Word2Vec(utils.SaveLoad):
     compatible with the original word2vec implementation via `save_word2vec_format()` and `load_word2vec_format()`.
 
     """
-    def __init__(self, sentences=None, size=100, alpha=0.025, window=5, min_count=5, seed=1, workers=1, min_alpha=0.0001, sg=1):
+    def __init__(self, sentences=None, size=100, alpha=0.025, window=5, min_count=5,
+        seed=1, workers=1, min_alpha=0.0001, sg=1, hs=1, negative=0, cbow_mean=0):
         """
         Initialize the model from an iterable of `sentences`. Each sentence is a
-        list of words (utf8 strings) that will be used for training.
+        list of words (unicode strings) that will be used for training.
 
         The `sentences` iterable can be simply a list, but for larger corpora,
         consider an iterable that streams the sentences directly from disk/network.
@@ -204,11 +245,16 @@ class Word2Vec(utils.SaveLoad):
         `seed` = for the random number generator.
         `min_count` = ignore all words with total frequency lower than this.
         `workers` = use this many worker threads to train the model (=faster training with multicore machines)
-
+        `hs` = if 1 (default), hierarchical sampling will be used for model training (else set to 0)
+        `negative` = if > 0, negative sampling will be used, the int for negative
+                specifies how many "noise words" should be drawn (usually between 5-20)
+        `cbow_mean` = if 0 (default), use the sum of the context word vectors. If 1, use the mean.
+                Only applies when cbow is used.
         """
         self.vocab = {}  # mapping from a word (string) to a Vocab object
         self.index2word = []  # map from a word's matrix index (int) to word (string)
         self.sg = int(sg)
+        self.table = None # for negative sampling --> this needs a lot of RAM! consider setting back to None before saving
         self.layer1_size = int(size)
         if size % 4 != 0:
             logger.warning("consider setting layer size to a multiple of 4 for greater performance")
@@ -218,10 +264,43 @@ class Word2Vec(utils.SaveLoad):
         self.min_count = min_count
         self.workers = workers
         self.min_alpha = min_alpha
+        self.hs = hs
+        self.negative = negative
+        self.cbow_mean = int(cbow_mean)
         if sentences is not None:
             self.build_vocab(sentences)
             self.train(sentences)
 
+    def make_table(self, table_size=100000000, power=0.75):
+        """
+        Create a table using stored vocabulary word counts for drawing random words in the negative
+        sampling training routines.
+
+        Called internally from `build_vocab()`.
+
+        """
+        logger.info("constructing a table with noise distribution from %i words" % len(self.vocab))
+        # table (= list of words) of noise distribution for negative sampling
+        vocab_size = len(self.index2word)
+        self.table = zeros(table_size, dtype=uint32)
+
+        if not vocab_size:
+            logger.warning("empty vocabulary in word2vec, is this intended?")
+            return
+
+        # compute sum of all power (Z in paper)
+        train_words_pow = float(sum([self.vocab[word].count**power for word in self.vocab]))
+        # go through the whole table and fill it up with the word indexes proportional to a word's count**power
+        widx = 0
+        # normalize count^0.75 by Z
+        d1 = self.vocab[self.index2word[widx]].count**power / train_words_pow
+        for tidx in xrange(table_size):
+            self.table[tidx] = widx
+            if 1.0 * tidx / table_size > d1:
+                widx += 1
+                d1 += self.vocab[self.index2word[widx]].count**power / train_words_pow
+            if widx >= vocab_size:
+                widx = vocab_size - 1
 
     def create_binary_tree(self):
         """
@@ -258,7 +337,7 @@ class Word2Vec(utils.SaveLoad):
     def build_vocab(self, sentences):
         """
         Build vocabulary from a sequence of sentences (can be a once-only generator stream).
-        Each sentence must be a list of utf8 strings.
+        Each sentence must be a list of unicode strings.
 
         """
         logger.info("collecting all words and their counts")
@@ -286,21 +365,27 @@ class Word2Vec(utils.SaveLoad):
                 self.vocab[word] = v
         logger.info("total %i word types after removing those with count<%s" % (len(self.vocab), self.min_count))
 
-        # add info about each word's Huffman encoding
-        self.create_binary_tree()
+        if self.hs:
+            # add info about each word's Huffman encoding
+            self.create_binary_tree()
+        if self.negative:
+            # build the table for drawing random words (for negative sampling)
+            self.make_table()
         self.reset_weights()
 
 
     def train(self, sentences, total_words=None, word_count=0, chunksize=100):
         """
         Update the model's neural weights from a sequence of sentences (can be a once-only generator stream).
-        Each sentence must be a list of utf8 strings.
+        Each sentence must be a list of unicode strings.
 
         """
         if FAST_VERSION < 0:
             import warnings
             warnings.warn("Cython compilation failed, training will be slow. Do you have Cython installed? `pip install cython`")
-        logger.info("training model with %i workers on %i vocabulary and %i features" % (self.workers, len(self.vocab), self.layer1_size))
+        logger.info("training model with %i workers on %i vocabulary and %i features, "
+            "using 'skipgram'=%s 'hierarchical softmax'=%s and 'negative sampling'=%s" %
+            (self.workers, len(self.vocab), self.layer1_size, self.sg, self.hs, self.negative))
 
         if not self.vocab:
             raise RuntimeError("you must first build vocabulary before training the model")
@@ -366,7 +451,10 @@ class Word2Vec(utils.SaveLoad):
         # randomize weights vector by vector, rather than materializing a huge random matrix in RAM at once
         for i in xrange(len(self.vocab)):
             self.syn0[i] = (random.rand(self.layer1_size) - 0.5) / self.layer1_size
-        self.syn1 = zeros((len(self.vocab), self.layer1_size), dtype=REAL)
+        if self.hs:
+            self.syn1 = zeros((len(self.vocab), self.layer1_size), dtype=REAL)
+        if self.negative:
+            self.syn1neg = zeros((len(self.vocab), self.layer1_size), dtype=REAL)
         self.syn0norm = None
 
 
@@ -380,19 +468,18 @@ class Word2Vec(utils.SaveLoad):
             logger.info("Storing vocabulary in %s" % (fvocab))
             with utils.smart_open(fvocab, 'wb') as vout:
                 for word, vocab in sorted(iteritems(self.vocab), key=lambda item: -item[1].count):
-                    vout.write("%s %s\n" % (word, vocab.count))
+                    vout.write(utils.to_utf8("%s %s\n" % (word, vocab.count)))
         logger.info("storing %sx%s projection weights into %s" % (len(self.vocab), self.layer1_size, fname))
         assert (len(self.vocab), self.layer1_size) == self.syn0.shape
         with utils.smart_open(fname, 'wb') as fout:
-            fout.write("%s %s\n" % self.syn0.shape)
+            fout.write(utils.to_utf8("%s %s\n" % self.syn0.shape))
             # store in sorted order: most frequent words at the top
             for word, vocab in sorted(iteritems(self.vocab), key=lambda item: -item[1].count):
-                word = utils.to_utf8(word)  # always store in utf8
                 row = self.syn0[vocab.index]
                 if binary:
-                    fout.write("%s %s\n" % (word, row.tostring()))
+                    fout.write(utils.to_utf8(word) + b" " + row.tostring())
                 else:
-                    fout.write("%s %s\n" % (word, ' '.join("%f" % val for val in row)))
+                    fout.write(utils.to_utf8("%s %s\n" % (word, ' '.join("%f" % val for val in row))))
 
 
     @classmethod
@@ -415,12 +502,12 @@ class Word2Vec(utils.SaveLoad):
             counts = {}
             with utils.smart_open(fvocab) as fin:
                 for line in fin:
-                    word, count = line.strip().split()
+                    word, count = utils.to_unicode(line).strip().split()
                     counts[word] = int(count)
 
         logger.info("loading projection weights from %s" % (fname))
-        with utils.smart_open(fname, 'rb') as fin:
-            header = fin.readline()
+        with utils.smart_open(fname) as fin:
+            header = utils.to_unicode(fin.readline())
             vocab_size, layer1_size = map(int, header.split())  # throws for invalid file format
             result = Word2Vec(size=layer1_size)
             result.syn0 = zeros((vocab_size, layer1_size), dtype=REAL)
@@ -431,11 +518,11 @@ class Word2Vec(utils.SaveLoad):
                     word = []
                     while True:
                         ch = fin.read(1)
-                        if ch == ' ':
-                            word = ''.join(word)
+                        if ch == b' ':
                             break
-                        if ch != '\n':  # ignore newlines in front of words (some binary files have newline, some not)
+                        if ch != b'\n':  # ignore newlines in front of words (some binary files have newline, some don't)
                             word.append(ch)
+                    word = utils.to_unicode(b''.join(word))
                     if counts is None:
                         result.vocab[word] = Vocab(index=line_no, count=vocab_size - line_no)
                     elif word in counts:
@@ -584,7 +671,7 @@ class Word2Vec(utils.SaveLoad):
         if getattr(self, 'syn0norm', None) is None or replace:
             logger.info("precomputing L2-norms of word weight vectors")
             if replace:
-                for i in range(self.syn0.shape[0]):
+                for i in xrange(self.syn0.shape[0]):
                     self.syn0[i, :] /= sqrt((self.syn0[i, :] ** 2).sum(-1))
                 self.syn0norm = self.syn0
                 if hasattr(self, 'syn1'):
@@ -620,8 +707,9 @@ class Word2Vec(utils.SaveLoad):
                     correct, correct + incorrect))
 
         sections, section = [], None
-        for line_no, line in enumerate(open(questions)):
+        for line_no, line in enumerate(utils.smart_open(questions)):
             # TODO: use level3 BLAS (=evaluate multiple questions at once), for speed
+            line = utils.to_unicode(line)
             if line.startswith(': '):
                 # a new section starts => store the old section
                 if section:
@@ -679,7 +767,8 @@ class BrownCorpus(object):
             fname = os.path.join(self.dirname, fname)
             if not os.path.isfile(fname):
                 continue
-            for line in open(fname):
+            for line in utils.smart_open(fname):
+                line = utils.to_unicode(line)
                 # each file line is a single sentence in the Brown corpus
                 # each token is WORD/POS_TAG
                 token_tags = [t.split('/') for t in line.split() if len(t.split('/')) == 2]
@@ -708,7 +797,7 @@ class Text8Corpus(object):
                         yield sentence
                     break
                 last_token = text.rfind(' ')  # the last token may have been split in two... keep it for the next iteration
-                words, rest = (text[:last_token].split(), text[last_token:].strip()) if last_token >= 0 else ([], text)
+                words, rest = (utils.to_unicode(text[:last_token]).split(), text[last_token:].strip()) if last_token >= 0 else ([], text)
                 sentence.extend(words)
                 while len(sentence) >= max_sentence_length:
                     yield sentence[:max_sentence_length]
@@ -716,18 +805,20 @@ class Text8Corpus(object):
 
 
 class LineSentence(object):
+    """Simple format: one sentence = one line; words already preprocessed and separated by whitespace."""
     def __init__(self, source):
-        """Simple format: one sentence = one line; words already preprocessed and separated by whitespace.
+        """
+        `source` can be either a string or a file object.
 
-        source can be either a string or a file object
-
-        Thus, one can use this for just plain files:
+        Example::
 
             sentences = LineSentence('myfile.txt')
 
-        Or for compressed files:
+        Or for compressed files::
+
+            sentences = LineSentence('compressed_text.txt.bz2')
+            sentences = LineSentence('compressed_text.txt.gz')
 
-            sentences = LineSentence(bz2.BZ2File('compressed_text.bz2'))
         """
         self.source = source
 
@@ -738,12 +829,12 @@ class LineSentence(object):
             # Things that don't have seek will trigger an exception
             self.source.seek(0)
             for line in self.source:
-                yield line.split()
+                yield utils.to_unicode(line).split()
         except AttributeError:
             # If it didn't work like a file, use it as a string filename
             with utils.smart_open(self.source) as fin:
                 for line in fin:
-                    yield line.split()
+                    yield utils.to_unicode(line).split()
 
 
 
